@@ -40,13 +40,16 @@ json_create_status_t;
 typedef struct json_create {
     /* The length of the input string. */
     int length;
-    unsigned char buffer[BUFSIZE];
+    unsigned char * buffer;
     /* Place to write the buffer to. */
     SV * output;
     /* Format for floating point numbers. */
     char * fformat;
     /* Memory leak counter. */
     int n_mallocs;
+    /* Handlers for objects and booleans. If there are no handlers,
+       this is zero (a NULL pointer). */
+    HV * handlers;
     /* Do any of the SVs have a Unicode flag? */
     unsigned int unicode : 1;
     /* Does the SV we're currently looking at have a Unicode flag? */
@@ -701,6 +704,81 @@ is_regexp (pTHX_ SV* sv)
     }							\
     return json_create_unknown_type
 
+//#define DEBUGOBJ
+
+static json_create_status_t
+json_create_handle_object (json_create_t * jc, SV * input, SV * r)
+{
+    const char * objtype;
+    /* The second argument to sv_reftype is true if we
+       look it up in the object table, false
+       otherwise. Undocumented, reported as
+       https://rt.perl.org/Ticket/Display.html?id=126469. */
+    objtype = sv_reftype (r, 1);
+    if (objtype) {
+	SV ** sv_ptr;
+	I32 olen;
+#ifdef DEBUGOBJ
+	fprintf (stderr, "Have found an object of type %s.\n", objtype);
+#endif
+	olen = strlen (objtype);
+	sv_ptr = hv_fetch (jc->handlers, objtype, olen, 0);
+	if (sv_ptr) {
+	    char * pv;
+	    STRLEN pvlen;
+	    pv = SvPV (*sv_ptr, pvlen);
+#ifdef DEBUGOBJ
+	    fprintf (stderr, "Have found a handler %s for %s.\n", pv, objtype);
+#endif
+	    if (pvlen == strlen ("bool") &&
+		strncmp (pv, "bool", 4) == 0) {
+		if (SvTRUE (r)) {
+		    ADD ("true");
+		}
+		else {
+		    ADD ("false");
+		}
+	    }
+	    else {
+		/* It's an object, it's in our handlers, but we don't
+	       have any code to deal with it, so we'll print an error
+	       and then stringify it. */
+		if (JCEH) {
+		    (*JCEH) (__FILE__, __LINE__, "Unhandled handler %s.\n",
+			     pv);
+		    goto nothandled;
+		}
+	    }
+	}
+	else {
+#ifdef DEBUGOBJ
+	    /* Leaving this debugging code here since this is liable
+	       to change a lot. */
+	    I32 hvnum;
+	    SV * s;
+	    char * key;
+	    I32 retlen;
+	    fprintf (stderr, "Nothing in handlers for %s.\n", objtype);
+	    hvnum = hv_iterinit (jc->handlers);
+
+	    fprintf (stderr, "There are %ld keys in handlers.\n", hvnum);
+	    while (1) {
+		s = hv_iternextsv (jc->handlers, & key, & retlen);
+		if (! s) {
+		    break;
+		}
+		fprintf (stderr, "%s: %s\n", key, SvPV_nolen (s));
+	    }
+#endif /* 0 */
+	    /* It's an object, but it's not in our handlers, so we'll
+	       just stringify it. */
+	nothandled:
+	    CALL (json_create_add_string (jc, input));
+	}
+    }
+    return json_create_ok;
+}
+
 /* This is the core routine, it is called recursively as hash values
    and array values containing array or hash references are
    handled. */
@@ -734,7 +812,15 @@ json_create_recursively (json_create_t * jc, SV * input)
 	    break;
 
 	case SVt_PVMG:
-	    CALL (json_create_add_string (jc, input));
+#ifdef DEBUGOBJ
+	    fprintf (stderr, "monkey monkey.\n");
+#endif 
+	    if (sv_isobject (input) && jc->handlers) {
+		CALL (json_create_handle_object (jc, input, r));
+	    }
+	    else {
+		CALL (json_create_add_string (jc, input));
+	    }
 	    break;
 
 	case SVt_PVGV:
@@ -829,11 +915,15 @@ json_create_recursively (json_create_t * jc, SV * input)
 static INLINE SV *
 json_create_run (json_create_t * jc, SV * input)
 {
+    unsigned char buffer[BUFSIZE];
+
     /* Set up all the transient variables for reading. */
 
     /* "jc.buffer" is dirty here, we have not initialized it, we are
        just writing to uninitialized stack memory. "jc.length" is the
        only thing we know is OK at this point. */
+    jc->buffer = buffer;
+
     jc->length = 0;
     /* Tell json_create_buffer_fill that it needs to allocate an
        SV. */
@@ -882,11 +972,21 @@ json_create_run (json_create_t * jc, SV * input)
 static SV *
 json_create (SV * input)
 {
-    json_create_t jc;
+    /* With all the options, this really needs to be blanked out. Thus
+       "buffer" is moved from being inside "jc" to being inside
+       "json_create_run" above. */
+    json_create_t jc = {0};
+
+    /* Set up the default options. */
+
     /* Floating point number format. */
     jc.fformat = 0;
     /* Escape slash. */
     jc.escape_slash = 0;
+
+    jc.unicode_escape_all = 0;
+    jc.handlers = 0;
+
     return json_create_run (& jc, input);
 }
 
@@ -944,6 +1044,15 @@ static json_create_status_t
 json_create_free (json_create_t * jc)
 {
     CALL (json_create_free_fformat (jc));
+
+    if (jc->handlers) {
+	SvREFCNT_dec ((SV *) jc->handlers);
+	jc->handlers = 0;
+	jc->n_mallocs--;
+    }
+
+    /* Finished, check we have no leaks before freeing. */
+
     jc->n_mallocs--;
     if (jc->n_mallocs != 0) {
 	fprintf (stderr, "%s:%d: n_mallocs = %d\n",
