@@ -7,7 +7,7 @@ require Exporter;
 );
 use warnings;
 use strict;
-our $VERSION = '0.06_01';
+our $VERSION = '0.06_02';
 require XSLoader;
 XSLoader::load ('JSON::Create', $VERSION);
 
@@ -35,4 +35,14 @@ sub set_fformat
     }
 }
 
+sub bool
+{
+    my ($obj, @list) = @_;
+    my %handlers;
+    for my $item (@list) {
+	$handlers{$item} = 'bool';
+    }
+    $obj->set_handlers (\%handlers);
+}
+
 1;
